.\" $Id: keeper.man,v 1.15 1997/03/26 14:23:08 esr Exp $
.TH KEEPER 1 "6 Mar 1997"
.AT 3
.SH NAME
keeper \- archivist's assistant for FTP & WWW archive sites
.SH SYNOPSIS
.B keeper [-r dir] [-s pref]
.PP
.SH THEORY
.PP
\fIKeeper\fR is an archivist's assistant for large Internet-accessible
software repositories.  Such sites need to be able to (a) file
submissions into an extensible topic tree, (b) associate descriptive
meta-information with each submission, and (c) maintain FTP and WWW
indices which make the archive contents and metainformation readily
browseable and searchable.
.PP
\fIKeeper\fR assumes that your archive setup is roughly like
Sunsite's.  That is, you have an FTP hierarchy that is WWW-accessible,
divided into a hierarchy of topic directories (the `topic
tree'). There is an incoming directory where submissions can be
dropped.  You need to match submissions with older versions, and
replace older submissions when appropriate while preserving as much
meta-information as possible.  You want to be able to automatically
maintain parallel FTP and HTML index files for each topic directory.
Some directories in the topic tree are actually mirror points for
archives on other sites and must be treaded as "read-only".
.PP
The topic knowledge in a keeper-maintained archive is expressed in two
ways: (1) the arrangement of the topic directories, and (2) !INDEX
files.  An !INDEX file contains one line of meta-information
(typically description info) for each non-document file in its
directory, including subdirectories.  An !INDEX file may also contain
a long-format description of its directory.  !INDEX files are directly
browseable by FTP and are used to automatically generate both long-
and short-format HTML index pages and FTP-able README files.
.PP
\fRKeeper\fR makes one other assumption about user-supplied
meta-information; that every submission comes accompanied with an LSM
(Linux Software Map) entry.  This is important for two reasons.
First, \fIkeeper\fR relies on the Author and Maintainer fields in the
LSM to generate email notifications when the status of a package
changes.  Second, requiring an LSM ensures that archive users will
always be easily able to get at package descriptions written by the
package maintainers themselves (as opposed to one-liners written by an
overworked archivist!)
.PP
.SH DESCRIPTION
.PP
\fIKeeper\fR itself is an dispatcher program resembling a
specialized shell.  You will normally use it in interactive mode, but
it has some batch options (documented below).
.PP
In the interactive mode, \fIkeeper's\fR view of the world is organized
around the idea of a "filegroup", a collection of archive and document
files that should be treated as a unit for archiving purposes.
\fIKeeper\fR is a tool for picking up filegroups, matching them
against other filegroups, and moving them between places in the
archive tree (initially from the incoming-submissions directory to a
topic directories and possibly later between topic directories).
\fIKeeper\fR also supports editing the topic tree itself.
.PP
The commands of interactive \fIkeeper\fR are fully documented in its
own on-line help; type `?' at the main prompt to get started.
.PP
\fIKeeper\fR has proper multi-user interlocks, so it can be used by
several collaborating archive maintainers simultaneously.
.PP
\fIKeeper\fR has batch modes triggered by command-line options:
.TP 5
-r <dir>
Rebuild the subtree with <dir> at its top (<dir> must be given
relative to the archive top).  To rebuild the whole archive, do `-r /'.
A rebuild checks all !INDEX files. prompts for missing descriptions,
and regenerates both long and short-form HTML index files in the
subtree.
.TP 5
-s
Stem.  Extract and display the stem of the filename argument; that is,
the prefix keeper would use to match it against other filenames.
(This option is intended for debugging and nay be removed from future
versions.)
.TP 5
-c
Policy check.  Check all LSMs and list every copying policy that
doesn't match GPL, BSD, or one of the known variants.
.PP
.SH INDEX AND DERIVED FILES
.PP
The maintainer-supplied information in a keeper archive is contained
in master index files, one per topic directory.  Each of these files
is named !INDEX (this name is chosen because it's quite unlikely to
be part of a package, and so it will sort to the top in an FTP
listing).
.PP
Each !INDEX file consists of four parts in order from the top: a header
line, an optional long desacription section, a delimiter line of
dashes, and a file-descriptions part.
.PP
The first line is used to identify the file and stamp it as
keeper-generated.
.PP
Any text after the first line but before before the line of dashes in
an !INDEX file is treated as a long directory description.  It is
preserved across index rebuilds, inserted in the header portion of the
long-form HTML index when that is rebuilt.  It is also used to
generate an FTPable README whenever there is an index rebuild.  If a
recognizable URL occurs in this text (that is, a word consisting of a
colon-terminated service prefix followed by //), it will be turned
into a live link in the HTML index.
.PP
After the line of dashes, each line is presumed to consist of a
filename, followed by whitespace, followed by one line of description.
There should be such a description for every package in the directory.
.PP
When an archive action (such as moving a package into a directory)
triggers an index rebuild, the contents of these sections is
reformatted into three different derived indexes; a long-form HTML
index called !INDEX/html, a short-form HTML file called
!INDEX.short.html, and an FTPable README file.
.PP
.SH ACTION NOTICES
.PP
\fIKeeper\fR tries to email a rejection notice with description of the
problems whenever you process an incoming filegroup with corrupt
archives or a missing or invalid LSM file.  It will prompt for an
author address if it can't get one from an LSM.  You may be able find
the author by looking for older versions in the archive or with a web
search.
.PP
\fIKeeper\fR tries to mail out action notices to the maintainers
whenever files are zapped out of a file group.  It prompts for a
one-line reason.  If it can't parse an email contact list out of LSM
files in the group, it will skip this step.
.PP
\fIKeeper\fR tries to mail out action notices to the maintainers
whenever a package is moved (either placed from the incoming directory
or moved withing the topic tree).  If it can't parse an email contact
list out of the LSM file it will skip this step.
.PP
\fIKeeper\fR tries to mail out action notices to the maintainers
whenever a package file description is changed. If it can't parse an
email contact list out of the LSM file it will skip this step.
.PP
\fIKeeper\fR can send a form letter complaining about a package's name
when asked to with the `yell' command.  See the on-line help for
the `yell' command for details.
.PP
Whenever \fIkeeper\fR sends out an email notice, the relevant
directory is checked against a watch list and possibly copied to
interested archive maintainers.  See the description of the watchlist
file below.
.PP
KEEPER AND MIRROR POINTS
.PP
\fIKeeper\fR can be given a list of configuration files which are
assumed to be in the format used by the Perl `mirror' package.  These
will be scanned for local mirror points.
.PP
In index files, a mirror point will be flagged with `/&' on the 
directory entry in its parent. 
.PP
Mirror points are otherwise made invisible in \fIkeeper\fR; you cannot
move files into or out of them, you cannot change directory to them,
and \fIkeeper\fR will not try to match incoming filenames in them
.PP
An attempt to delete a mirror point directory (or any directory
above it) will be detected and rejected.
.PP
An attempt to move a mirror point directory (or any directory
above it) will be detected and rejected. 
.PP
These two files are automatically generated in the archive's root directory.
.SH FILES
.TP 5
NEW
Each dispatch of a file group from the incoming directory to a topic
directory is logged here.
.TP 5
NEW.html
HTMLized version of NEW intended to be periodically put at an
accessible URL in the archive.
.TP 5
LOG
Each edit of the topic tree, and each filegroup move internal to the
topic tree, is logged here.
.PP
All these files are in the `shared' directory configured near the
head of the source.  All are updated automatically.
.TP 5
watchlist
A list of watch-list assignments, one per line.  Each consists of a
first whitespace-delimited field which is interpreted as an email
address.  Subsequent whitespace-delimited fields are Perl regexps to be
matched against directory names.  Each time an email notice is sent,
the directory it is relevant to is checked against the watch regexps
and blind-copied to any emal address with a matching regexp.
.TP 5
files
A list of all files in the archive.  This is used for prefix-matching
incoming filegroups with earlier versions already in the archive.
.TP 5
directories
A list of all topic directories.  Each line consists of a directory
name, followed by a tab, followed by the directory's description (as
extracted from its parent's !INDEX).  This is used to check path 
arguments, and to display candidate directories natching a regular
expression.
.TP 5
LOCK
Exclusion lock file.
.TP 5
LOCK.ASSERTED
Link to exclusion lock file.  Only exists when some copy of keeper has
asserted a lock.
.SH BUGS
\fIKeeper\fR is a Perl script.  The current version has file locations
appropriate for the Linux Sunsite archive wired into its configuration
section at the head of the source file.  To configure it, you must
edit the configuration section.  (This will change in future
versions.)
.PP
Keeper will sometimes choke and die on command arguments that are not
valid Perl regexps (this happens most commonly with cd).  It is a good
idea to quote the non-alphabetic characters in, e.g., directories like
devel/lang/c++.
.PP
In principle, when \fIkeeper\fR can detect that a directory is a
symlink to a mirror point elsewhere, it could permit the link to
be moved or deleted while still suppressing other operations.  This
will be supported in future versions.
.PP
For speed, \fIkeeper\fR does not rebuild its `files' and `directories'
lists after every file dispatch from incoming.  It is thus
theoretically possible that two versions of a package could come in
in such quick succession that \fIkeeper\fR would fail to match the
second against the first, resulting in the first's not being deleted.
.SH AUTHOR
Eric S. Raymond <esr@thyrsus.com>.  Uses some code originated by Erik
Troan <ewt@redhat.com>.
